#include "SynthInput.h"
#include "SaveBMP.h"
#include <iostream>

using namespace std;

SynthInput::SynthInput()
{
  // Get the dimensions of the desktop.
  HWND hDesktop = GetDesktopWindow();
  LastActivatedWindow = hDesktop;
  GetWindowRect(hDesktop, &LastActivatedWindowRect); 
  DesktopWidth = LastActivatedWindowRect.right;
  DesktopHeight = LastActivatedWindowRect.bottom;
  // Compute the scale factors
  ScaleX = 65535.0f / DesktopWidth;
  ScaleY = 65535.0f / DesktopHeight;
}

SynthInput::~SynthInput()
{
}

int SynthInput::GetDesktopWidth() 
{
  return DesktopWidth;
}

int SynthInput::GetDesktopHeight() 
{
  return DesktopHeight;
}

bool SynthInput::MoveMouseTo(int MoveToX, int MoveToY)
{
     INPUT Input;
     Input.type = INPUT_MOUSE;
     Input.mi.dx = (LONG )(MoveToX * ScaleX);
     Input.mi.dy = (LONG )(MoveToY * ScaleY);
     Input.mi.dwFlags = MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MOVE;
     Input.mi.time = 0;
     Input.mi.dwExtraInfo = 0;

     int n = SendInput(1, &Input, sizeof(INPUT));
     return n > 0;
}

bool SynthInput::LeftClick()
{
     INPUT Input[2];
     Input[0].type = INPUT_MOUSE;
     Input[0].mi.dx = 0;
     Input[0].mi.dy = 0;
     Input[0].mi.dwFlags = MOUSEEVENTF_LEFTDOWN;
     Input[0].mi.time = 0;
     Input[0].mi.dwExtraInfo = 0;

     Input[1].type = INPUT_MOUSE;
     Input[1].mi.dx = 0;
     Input[1].mi.dy = 0;
     Input[1].mi.dwFlags = MOUSEEVENTF_LEFTUP;
     Input[1].mi.time = 0;
     Input[1].mi.dwExtraInfo = 0;
     
     int n = SendInput(2, Input, sizeof(INPUT));
     return n > 0;
}

bool SynthInput::RightClick()
{
     INPUT Input[2];
     Input[0].type = INPUT_MOUSE;
     Input[0].mi.dx = 0;
     Input[0].mi.dy = 0;
     Input[0].mi.dwFlags = MOUSEEVENTF_RIGHTDOWN;
     Input[0].mi.time = 0;
     Input[0].mi.dwExtraInfo = 0;

     Input[1].type = INPUT_MOUSE;
     Input[1].mi.dx = 0;
     Input[1].mi.dy = 0;
     Input[1].mi.dwFlags = MOUSEEVENTF_RIGHTUP;
     Input[1].mi.time = 0;
     Input[1].mi.dwExtraInfo = 0;
     
     int n = SendInput(2, Input, sizeof(INPUT));
     return n > 0;
}

bool SynthInput::TypeVirtualKey(WORD Key)
{
     INPUT Input[2];
     Input[0].type = INPUT_KEYBOARD;
     Input[0].ki.wVk = Key;
     Input[0].ki.wScan = 0;
     Input[0].ki.dwFlags = 0;
     Input[0].ki.time = 0;     
     Input[0].ki.dwExtraInfo = 0;
     
     Input[1].type = INPUT_KEYBOARD;
     Input[1].ki.wVk = Key;
     Input[1].ki.wScan = 0;
     Input[1].ki.dwFlags = KEYEVENTF_KEYUP;
     Input[1].ki.time = 0;     
     Input[1].ki.dwExtraInfo = 0;

     int n = SendInput(2, Input, sizeof(INPUT));
     return n == 2;
}

bool SynthInput::ActivateExactWindow(char *Title)
{
    HWND windowHandle = FindWindow(0, Title);
    if (windowHandle == NULL)
    {
      return false;
    }
    else
    {
      SetForegroundWindow(windowHandle);
      SetActiveWindow(windowHandle);
	  LastActivatedWindow = windowHandle;
      GetWindowRect(windowHandle, &LastActivatedWindowRect);
      return true;
    } 
}

BOOL CALLBACK EnumCallbackProc(HWND hwnd, LPARAM lParam)
{
     char Name[256];
     int NameSize = 256;
     SynthInput *oInput = (SynthInput *)lParam;
     
     GetWindowText(hwnd, Name, NameSize);
     if (strstr(Name, oInput->PartialWindowName))
     {
        //cerr << "Found Window: " << Name << endl;
        oInput->EnumResult = true;
        SetForegroundWindow(hwnd);
        SetActiveWindow(hwnd);
		ShowWindow(hwnd, SW_SHOW);
		oInput->LastActivatedWindow = hwnd;
        GetWindowRect(hwnd, &(oInput->LastActivatedWindowRect));
		//cerr << "Width: " << (oInput->LastActivatedWindowRect.right - oInput->LastActivatedWindowRect.left);
		//cerr << ", Height: " << (oInput->LastActivatedWindowRect.bottom - oInput->LastActivatedWindowRect.top) << endl;
        return FALSE;
     }
     return TRUE;
}

bool SynthInput::ActivateWindow(char *PartialTitle)
{
    EnumResult = false;
    strcpy(PartialWindowName, PartialTitle);
    EnumWindows(EnumCallbackProc, (LPARAM )this);
    return EnumResult;
}

RECT SynthInput::GetLastActivatedWindowRect()
{
     return LastActivatedWindowRect;
}

HWND SynthInput::GetLastActivatedWindow()
{
	return LastActivatedWindow;
}

HBITMAP SynthInput::CaptureLastActivatedWindow()
{
	Sleep(1);
	HDC LastActivatedDC = GetWindowDC(LastActivatedWindow);
	HDC CaptureDC = CreateCompatibleDC(LastActivatedDC);

	int Width = LastActivatedWindowRect.right - LastActivatedWindowRect.left;
	int Height = LastActivatedWindowRect.bottom - LastActivatedWindowRect.top;
	HBITMAP CaptureBitmap = CreateCompatibleBitmap(LastActivatedDC, Width, Height);

	SelectObject(CaptureDC, CaptureBitmap);

	//BitBlt(CaptureDC, 0,0, Width,Height, LastActivatedDC, 0,0, SRCCOPY|CAPTUREBLT);
	BitBlt(CaptureDC, 0,0, Width,Height, LastActivatedDC, 0,0, SRCCOPY);

	ReleaseDC(LastActivatedWindow, LastActivatedDC);
	DeleteDC(CaptureDC);
	DeleteDC(LastActivatedDC);

	//CreateBMPFile("LastCapture.bmp", CaptureBitmap);

	return CaptureBitmap;
}

HBITMAP SynthInput::CaptureLastActivatedWindow(int x, int y, int Width, int Height)
{
	Sleep(1);
	HDC LastActivatedDC = GetWindowDC(LastActivatedWindow);
	HDC CaptureDC = CreateCompatibleDC(LastActivatedDC);

	HBITMAP CaptureBitmap = CreateCompatibleBitmap(LastActivatedDC, Width, Height);

	SelectObject(CaptureDC, CaptureBitmap);

	//BitBlt(CaptureDC, 0,0, Width,Height, LastActivatedDC, x,y, SRCCOPY|CAPTUREBLT);
	BitBlt(CaptureDC, 0,0, Width,Height, LastActivatedDC, x,y, SRCCOPY);

	ReleaseDC(LastActivatedWindow, LastActivatedDC);
	DeleteDC(CaptureDC);
	DeleteDC(LastActivatedDC);

	return CaptureBitmap;
}

void SynthInput::CaptureLastActivatedWindow(char *FileName)
{
	HBITMAP CaptureBitmap = CaptureLastActivatedWindow();
	CreateBMPFile(FileName, CaptureBitmap);	
	DeleteObject(CaptureBitmap);
}

void SynthInput::CaptureLastActivatedWindow(char *FileName, int x, int y, int Width, int Height)
{
	HBITMAP CaptureBitmap = CaptureLastActivatedWindow(x,y, Width,Height);
	CreateBMPFile(FileName, CaptureBitmap);	
	DeleteObject(CaptureBitmap);
}

unsigned char *SynthInput::BitmapToArray(HBITMAP hBitmap, int *Width, int *Height)
{
	HDC hdc = GetDC(NULL);

	// First get the information on the bitmap,
	BITMAPINFO BitmapInfo = {0};
	BitmapInfo.bmiHeader.biSize = sizeof(BITMAPINFO);
	PBITMAPINFO pBitmapInfo = &BitmapInfo;

	int Result = GetDIBits(hdc, hBitmap, 0, 0, NULL, pBitmapInfo, DIB_RGB_COLORS);
	if (Result == 0)
	{
		cerr << "Could not fill the BitmapInfo struct!" << endl;
		return NULL;
	}
	*Width = (int )BitmapInfo.bmiHeader.biWidth;
	*Height = (int )BitmapInfo.bmiHeader.biHeight;

	int AdditionalMemory = 0;
	switch (BitmapInfo.bmiHeader.biBitCount)
	{
		case 1:
			AdditionalMemory = 1 * sizeof(RGBQUAD);
			break;
		case 4:
			AdditionalMemory = 15 * sizeof(RGBQUAD);
			break;
		case 8:
			AdditionalMemory = 255 * sizeof(RGBQUAD);
			break;
		case 16:
		case 32:
			AdditionalMemory = 2 * sizeof(RGBQUAD);
	}

	unsigned char *pByte = NULL;
	if (AdditionalMemory)
	{
		pByte = new unsigned char[sizeof(BITMAPINFO) + AdditionalMemory];
		if (pByte)
		{
			memset(pByte, 0, sizeof(BITMAPINFO) + AdditionalMemory);
			memcpy(pByte, &BitmapInfo, sizeof(BITMAPINFO));
			pBitmapInfo = (PBITMAPINFO )pByte;
		}
	}
  
	unsigned char *Array = new unsigned char[BitmapInfo.bmiHeader.biSizeImage];

	// Now get the buffer itself.
	Result = GetDIBits(hdc, hBitmap, 0, pBitmapInfo->bmiHeader.biHeight, Array, pBitmapInfo, DIB_RGB_COLORS);
	if (pByte)
	{
		delete[] pByte;
	}

	//ReleaseDC(LastActivatedWindow, hdc);
	ReleaseDC(NULL, hdc);

	// Final processing: convert everything to 3-byte RGB data buffers.
	if (BitmapInfo.bmiHeader.biBitCount == 32)
	{
		int x,y, SrcOfs = 0, DestOfs = 0;
		int W = *Width, H = *Height;
		unsigned char *Temp = new unsigned char[W * H *3];
		for (y = 0; y < H; y++)
			for (x = 0; x < W; x++)
			{
				Temp[DestOfs] = Array[SrcOfs];
				DestOfs++; SrcOfs++;

				Temp[DestOfs] = Array[SrcOfs];
				DestOfs++; SrcOfs++;
				
				Temp[DestOfs] = Array[SrcOfs];
				DestOfs++; SrcOfs++;

				SrcOfs++;
			}
		delete[] Array;
		Array = Temp;
	}

	return Array;
}

bool SynthInput::EqualBitmaps(HBITMAP hBitmap1, HBITMAP hBitmap2)
{
	int W1,H1, W2,H2;
	unsigned char *Data1 = BitmapToArray(hBitmap1, &W1,&H1);
	unsigned char *Data2 = BitmapToArray(hBitmap2, &W2,&H2);
	if (W1 != W2 || H1 != H2)
	{
		return false;
	}

	bool Result = (0 == memcmp(Data1, Data2, W1*H1*3));
	delete[] Data1;
	delete[] Data2;
	return Result;
}

HBITMAP SynthInput::LoadBitmap(char *FileName)
{
	//return (HBITMAP )LoadImage(NULL, FileName, IMAGE_BITMAP, 0,0, LR_DEFAULTSIZE | LR_LOADFROMFILE | LR_CREATEDIBSECTION);
	return (HBITMAP )LoadImage(NULL, FileName, IMAGE_BITMAP, 0,0, LR_DEFAULTSIZE | LR_LOADFROMFILE);
}

unsigned char **SynthInput::LoadAsGrayMatrix(char *FileName, int *Width, int *Height)
{
	HANDLE hBitmap = LoadImage(NULL, FileName, IMAGE_BITMAP, 0,0, LR_DEFAULTSIZE | LR_LOADFROMFILE);
	if (NULL == hBitmap) 
	{
		*Width = *Height = 0;
		return NULL;
	}
	unsigned char **Data = BitmapToGrayMatrix((HBITMAP )hBitmap, Width, Height);
	DeleteObject(hBitmap);
	return Data;
}

unsigned char **SynthInput::BitmapToGrayMatrix(HBITMAP hBitmap, int *Width, int *Height)
{
	HDC hdc = GetDC(NULL);

	// First get the information on the bitmap.
	BITMAPINFO BitmapInfo = {0};
	BitmapInfo.bmiHeader.biSize = sizeof(BITMAPINFO);
	PBITMAPINFO pBitmapInfo = &BitmapInfo;

	int Result = GetDIBits(hdc, hBitmap, 0, 0, NULL, pBitmapInfo, DIB_RGB_COLORS);
	if (Result == 0)
	{
		cerr << "Could not fill the BitmapInfo struct!" << endl;
		return NULL;
	}
	*Width = (int )BitmapInfo.bmiHeader.biWidth;
	*Height = (int )BitmapInfo.bmiHeader.biHeight;

	int AdditionalMemory = 0;
	switch (BitmapInfo.bmiHeader.biBitCount)
	{
		case 1:
			AdditionalMemory = 1 * sizeof(RGBQUAD);
			break;
		case 4:
			AdditionalMemory = 15 * sizeof(RGBQUAD);
			break;
		case 8:
			AdditionalMemory = 255 * sizeof(RGBQUAD);
			break;
		case 16:
		case 32:
			AdditionalMemory = 2 * sizeof(RGBQUAD);
	}

	unsigned char *pByte = NULL;
	if (AdditionalMemory)
	{
		pByte = new unsigned char[sizeof(BITMAPINFO) + AdditionalMemory];
		if (pByte)
		{
			memset(pByte, 0, sizeof(BITMAPINFO) + AdditionalMemory);
			memcpy(pByte, &BitmapInfo, sizeof(BITMAPINFO));
			pBitmapInfo = (PBITMAPINFO )pByte;
		}
	}
  
	unsigned char *Array = new unsigned char[BitmapInfo.bmiHeader.biSizeImage];

	// Now get the buffer itself.
	Result = GetDIBits(hdc, hBitmap, 0, pBitmapInfo->bmiHeader.biHeight, Array, pBitmapInfo, DIB_RGB_COLORS);
	if (pByte)
	{
		delete[] pByte;
	}

	ReleaseDC(NULL, hdc);

	// Final processing: convert everything to a grayscale matrix.
	bool Skip = (BitmapInfo.bmiHeader.biBitCount == 32);
	int x,y, SrcOfs = 0;
	int W = *Width, H = *Height;
	unsigned char R,G,B, g;
	unsigned char **Data = new unsigned char *[H];
	for (y = H - 1; y >= 0; y--)
	{
		Data[y] = new unsigned char[W];
		for (x = 0; x < W; x++)
		{
			B = Array[SrcOfs++];
			G = Array[SrcOfs++];
			R = Array[SrcOfs++];
			if (Skip) SrcOfs++;
			g = (R + G + B) / 3;
			Data[y][x] = g;
		}
	}
	delete[] Array;

	return Data;
}

bool SynthInput::CompareMatricesAt(unsigned char **MovingData1, int Width1, int Height1, unsigned char **Data2, int Width2, int Height2, int x0, int y0)
{
	int x1,y1, x2,y2;
	for (y1 = 0, y2 = y0; y1 < Height1; y1++, y2++)
	{
		if (y2 >= Height2) 
			return false;
		for (x1 = 0, x2 = x0; x1 < Width1; x1++, x2++)
		{
			if (x2 > Width2) 
				return false;
			if (MovingData1[y1][x1] != Data2[y2][x2]) 
				return false;
		}
	}
	return true;
}
