#ifndef SYNTH_INPUT_H
#define SYNTH_INPUT_H

#define _WIN32_WINNT 0x0501
#include <windows.h>

class SynthInput
{
private:
	  ULONG GdiPlusToken;
      int DesktopWidth, DesktopHeight;
      float ScaleX, ScaleY;
      char PartialWindowName[256];
      bool EnumResult;
      RECT LastActivatedWindowRect;
	  HWND LastActivatedWindow;
      
public:
      SynthInput();
	  ~SynthInput();

      int GetDesktopWidth();
      int GetDesktopHeight();
      
      bool MoveMouseTo(int MoveToX, int MoveToY);
      bool LeftClick();
	  bool RightClick();
      bool TypeVirtualKey(WORD Key);
      
      // Activates a window given an exact title match.
      bool ActivateExactWindow(char *Title);
      // Activates a window given a partial window title.
      bool ActivateWindow(char *PartialTitle);
      friend BOOL CALLBACK EnumCallbackProc(HWND hwnd, LPARAM lParam);
      
      RECT GetLastActivatedWindowRect();
	  HWND GetLastActivatedWindow();

	  // This function returns an HBITMAP handle. Do please remember to DeleteObject it after using it.
	  HBITMAP CaptureLastActivatedWindow();
	  HBITMAP CaptureLastActivatedWindow(int x, int y, int Width, int Height);
	  void CaptureLastActivatedWindow(char *FileName);
	  void CaptureLastActivatedWindow(char *FileName, int x, int y, int Width, int Height);
	  unsigned char *BitmapToArray(HBITMAP hBitmap, int *Width, int *Height);
	  bool EqualBitmaps(HBITMAP hBitmap1, HBITMAP hBitmap2);
	  HBITMAP LoadBitmap(char *FileName);

	  unsigned char **BitmapToGrayMatrix(HBITMAP hBitmap, int *Width, int *Height);
	  unsigned char **LoadAsGrayMatrix(char *FileName, int *Width, int *Height);
	  bool CompareMatricesAt(unsigned char **MovingData1, int Width1, int Height1, unsigned char **Data2, int Width2, int Height2, int x0, int y0);
};

#endif
