/* No copyright whatsoever. Do whatever you want with this code. Use it and
   abuse it just as long as you don't blame me for anything bad that might
   happen to you, your pets, your grandma or your significant other.

   Having said that, you may have fun with this code!
*/

#include "WinMineReader.h"
#include "ConsoleWriter.h"
#include <windows.h>
#include <process.h>
#include <SWI-Prolog.h>

//////////////////////////////////////////////////////////////////////////////
#define APP_CLASS_NAME "WinMinePlayerPlugin"

void RegisterPrologPredicates();

HWND ConsoleWindow;
WinMineReader *oReader = NULL;

// Global vars
// The reference to the current module (DLL).
HINSTANCE hinstDLLGlobal;

BOOL OnCreate(HWND hwnd) {
  return TRUE;
}

void OnDestroy() {  
  FreeConsole();  
  UnregisterClass(APP_CLASS_NAME, hinstDLLGlobal);
}

void FindConsoleWindow() {
	// In this section, we create a console window with a specific title...
	char *Title = "WinMine Player Plugin Console";
	SetConsoleTitle(Title);
	ConsoleWindow = NULL;
	// ... then we find that console window by its title...
	while (ConsoleWindow == NULL)
		ConsoleWindow = FindWindowEx(NULL, NULL, NULL, Title);

	// ... we wouldn't want the close button on the console to be active, so we erase it...
	HMENU hmenu = GetSystemMenu( ConsoleWindow, FALSE );
    DeleteMenu( hmenu, SC_CLOSE, MF_BYCOMMAND );
    
	// ... and then, we get a handle to the console's standard output...
    hConsoleErr = GetStdHandle( STD_ERROR_HANDLE );

	// ... so we can write to it using the "cons" object defined in ConsoleWriter.h
	cons << "WinMine Player Plugin Console by Leo Reyes, 2008.\n";
	cons << "Thought of the day:\n\n";
	cons << "The sum of the square of your cathetes makes my hypothenuse go rectangle.\n\n";
	cons << "The Cow Hath Spoken!\n";
}

// DLL Entry and Exit Points
extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD dwReason, LPVOID lpRes) {
	switch(dwReason) {
		case DLL_PROCESS_ATTACH: {
			// This is where the fun begins. First, we save the reference to the current module (DLL)...
		    hinstDLLGlobal = hinstDLL;  

			// ... then we create a console...
			AllocConsole();
			// ... and get its output handle...
            FindConsoleWindow();

            oReader = new WinMineReader("Buscam");

			// ... and finally, we register some foreign predicates in the host Prolog engine.
			RegisterPrologPredicates();
 		    break;
        }
		case DLL_THREAD_ATTACH:	
		    break;
		case DLL_THREAD_DETACH:
            break;
		case DLL_PROCESS_DETACH:
			// One day, I'm gonna hump an Alpaca real hard.
            delete oReader;
		    OnDestroy();			
			break;
	}
	return TRUE;
}

foreign_t pl_UpdateMineField()
{
  if (NULL == oReader) {
    cons << "oReader == NULL. Something failed during initialization.\n";
    PL_fail;
  }

  oReader->UpdateMineField();
  PL_succeed;
}

foreign_t pl_ReadCell(term_t Row, term_t Col, term_t Element) {
  int iRow, iCol;
  PL_get_integer(Row, &iRow);
  PL_get_integer(Col, &iCol);

  if (iRow >= 0 && iRow < oReader->GetRows() &&
      iCol >= 0 && iCol < oReader->GetCols())
  {
    int Symbol = oReader->MineField[iRow][iCol];
    if (Symbol >= 0 && Symbol <= 9)
    {
      PL_unify_integer(Element, Symbol);
    }
    else
    {
      switch (oReader->MineField[iRow][iCol])
      {
        case SYMBOL_BOMB : PL_unify_atom_chars(Element, "bomb"); break;
        case SYMBOL_CROSS_BOMB : PL_unify_atom_chars(Element, "crossedbomb"); break;
        case SYMBOL_MINE_MARK : PL_unify_atom_chars(Element, "minemark"); break;
        case SYMBOL_NOT_CLICKED : PL_unify_atom_chars(Element, "notclicked"); break;
        case SYMBOL_QUESTION_MARK : PL_unify_atom_chars(Element, "qmark"); break;
        case SYMBOL_QUESTION_MARK_DEPRESSED : PL_unify_atom_chars(Element, "qmarkdepressed"); break;
        case SYMBOL_RED_BOMB : PL_unify_atom_chars(Element, "redbomb"); break;
        case SYMBOL_UNKNOWN : PL_unify_atom_chars(Element, "unknown"); break;
        default :
                PL_unify_atom_chars(Element, "error"); break;
      }
    }
    PL_succeed;
  }
  else
  {
    cons << "[" << iRow << ", " << iCol << "] out of boundaries\n";
    PL_fail;
  }
}

foreign_t pl_GetSize(term_t Rows, term_t Cols)
{
  if (NULL == oReader)
  {
     cons << "oReader == NULL. Error during initialization?\n";
     PL_fail;
  }  
  PL_unify_integer(Rows, oReader->GetRows());
  PL_unify_integer(Cols, oReader->GetCols());
  PL_succeed;
}

foreign_t pl_GetGameStatus(term_t Status, term_t NumberOfMines, term_t Seconds)
{
  if (NULL == oReader)
  {
     cons << "oReader == NULL. Error during initialization?\n";
     PL_fail;
  }

  switch (oReader->FaceStatus)
  {
    case FACE_FINISH :
      PL_unify_atom_chars(Status, "finished");
      break;
    case FACE_NOT_OK :
      PL_unify_atom_chars(Status, "notok");
      break;
    case FACE_OK :
      PL_unify_atom_chars(Status, "ok");
      break;
    case FACE_UNKNOWN :
      PL_unify_atom_chars(Status, "unknown");
      break;
    default:
      PL_unify_atom_chars(Status, "error");
      break;
  }

  PL_unify_integer(Seconds, oReader->Seconds);
  PL_unify_integer(NumberOfMines, oReader->NumberOfMines);
  PL_succeed;
}

foreign_t pl_MoveMouseTo(term_t Row, term_t Col)
{
  if (NULL == oReader)
  {
     cons << "oReader == NULL. Error during initialization?\n";
     PL_fail;
  }

  int iRow, iCol;
  PL_get_integer(Row, &iRow);
  PL_get_integer(Col, &iCol);
  if (iRow >= 0 && iRow < oReader->GetRows() &&
      iCol >= 0 && iCol < oReader->GetCols())
  {
    oReader->MoveMouseTo(iRow, iCol);
    PL_succeed;
  }
  else
  {
    PL_fail;
  }  
}

foreign_t pl_LeftClick()
{
  if (NULL == oReader)
  {
     cons << "oReader == NULL. Error during initialization?\n";
     PL_fail;
  }
  oReader->LeftClick();
  PL_succeed;
}

foreign_t pl_RightClick()
{
  if (NULL == oReader)
  {
     cons << "oReader == NULL. Error during initialization?\n";
     PL_fail;
  }
  oReader->RightClick();
  PL_succeed;
}

foreign_t pl_StartNewGame()
{
  if (NULL == oReader)
  {
     cons << "oReader == NULL. Error during initialization?\n";
     PL_fail;
  }
  oReader->SendKey(VK_F2);
  PL_succeed;
}

// The foreign predicates are registered here.
void RegisterPrologPredicates() {
  cons << "Registering prolog predicates...\n";

  if (PL_register_foreign("update_mine_field", 0, (void*)pl_UpdateMineField, 0)) {
    cons << "update_mine_field/0 registered successfully\n";
  }

  if (PL_register_foreign("get_size", 2, (void*)pl_GetSize, 0)) {
    cons << "get_size/2 [get_size(-Rows, -Cols)] registered successfully\n";
  }

  if (PL_register_foreign("read_cell", 3, (void*)pl_ReadCell, 0)) {
    cons << "read_cell/3 [read_cell(+Row, +Col, -Symbol)] registered successfully\n";
  }

  if (PL_register_foreign("game_status", 3, (void*)pl_GetGameStatus, 0)) {
    cons << "game_status/3 [game_status(-Status, -Mines, -Seconds)] registered successfully\n";
  }

  if (PL_register_foreign("move_mouse", 2, (void*)pl_MoveMouseTo, 0)) {
    cons << "move_mouse/2 [move_mouse(+Row, +Col)] registered successfully\n";
  }

  if (PL_register_foreign("left_click", 0, (void*)pl_LeftClick, 0)) {
    cons << "left_click/0 registered successfully\n";
  }

  if (PL_register_foreign("right_click", 0, (void*)pl_RightClick, 0)) {
    cons << "right_click/0 registered successfully\n";
  }

  if (PL_register_foreign("start_new_game", 0, (void*)pl_StartNewGame, 0)) {
    cons << "start_new_game/0 registered successfully\n";
  }
}
