#include "WinMineReader.h"
#include <iostream>

using namespace std;

WinMineReader::WinMineReader(char *winMineWindowName)
{
	strcpy(WinMineWindowName, winMineWindowName);

	int W,H;
	SquareSymbols[SYMBOL_0] = oInput.LoadAsGrayMatrix("0.bmp", &W,&H);
	SquareSymbols[SYMBOL_1] = oInput.LoadAsGrayMatrix("1.bmp", &W,&H);
	SquareSymbols[SYMBOL_2] = oInput.LoadAsGrayMatrix("2.bmp", &W,&H);
	SquareSymbols[SYMBOL_3] = oInput.LoadAsGrayMatrix("3.bmp", &W,&H);
	SquareSymbols[SYMBOL_4] = oInput.LoadAsGrayMatrix("4.bmp", &W,&H);
	SquareSymbols[SYMBOL_5] = oInput.LoadAsGrayMatrix("5.bmp", &W,&H);
	SquareSymbols[SYMBOL_6] = oInput.LoadAsGrayMatrix("6.bmp", &W,&H);
	SquareSymbols[SYMBOL_7] = oInput.LoadAsGrayMatrix("7.bmp", &W,&H);
	SquareSymbols[SYMBOL_8] = oInput.LoadAsGrayMatrix("8.bmp", &W,&H);
	SquareSymbols[SYMBOL_NOT_CLICKED] = oInput.LoadAsGrayMatrix("Unmarked.bmp", &W,&H);

	SquareSymbols[SYMBOL_BOMB] = oInput.LoadAsGrayMatrix("Bomb.bmp", &W,&H);
	SquareSymbols[SYMBOL_RED_BOMB] = oInput.LoadAsGrayMatrix("RedBomb.bmp", &W,&H);
	SquareSymbols[SYMBOL_CROSS_BOMB] = oInput.LoadAsGrayMatrix("CrossBomb.bmp", &W,&H);
	SquareSymbols[SYMBOL_MINE_MARK] = oInput.LoadAsGrayMatrix("Mark.bmp", &W,&H);
	SquareSymbols[SYMBOL_QUESTION_MARK_DEPRESSED] = oInput.LoadAsGrayMatrix("QMark.bmp", &W,&H);
	SquareSymbols[SYMBOL_QUESTION_MARK] = oInput.LoadAsGrayMatrix("QMarkOver.bmp", &W,&H);

	// Load the digits
	DigitSymbols[DIGIT_0] = oInput.LoadAsGrayMatrix("Digit0.bmp", &W, &H);
	DigitSymbols[DIGIT_1] = oInput.LoadAsGrayMatrix("Digit1.bmp", &W, &H);
	DigitSymbols[DIGIT_2] = oInput.LoadAsGrayMatrix("Digit2.bmp", &W, &H);
	DigitSymbols[DIGIT_3] = oInput.LoadAsGrayMatrix("Digit3.bmp", &W, &H);
	DigitSymbols[DIGIT_4] = oInput.LoadAsGrayMatrix("Digit4.bmp", &W, &H);
	DigitSymbols[DIGIT_5] = oInput.LoadAsGrayMatrix("Digit5.bmp", &W, &H);
	DigitSymbols[DIGIT_6] = oInput.LoadAsGrayMatrix("Digit6.bmp", &W, &H);
	DigitSymbols[DIGIT_7] = oInput.LoadAsGrayMatrix("Digit7.bmp", &W, &H);
	DigitSymbols[DIGIT_8] = oInput.LoadAsGrayMatrix("Digit8.bmp", &W, &H);
	DigitSymbols[DIGIT_9] = oInput.LoadAsGrayMatrix("Digit9.bmp", &W, &H);
	DigitSymbols[DIGIT_MINUS] = oInput.LoadAsGrayMatrix("DigitMinus.bmp", &W, &H);
	DigitSymbols[DIGIT_NULL] = oInput.LoadAsGrayMatrix("DigitNull.bmp", &W, &H);

	FaceSymbols[FACE_OK] = oInput.LoadAsGrayMatrix("FaceOK.bmp", &W,&H);
	FaceSymbols[FACE_NOT_OK] = oInput.LoadAsGrayMatrix("FaceNotOK.bmp", &W,&H);
	FaceSymbols[FACE_FINISH] = oInput.LoadAsGrayMatrix("FaceFinish.bmp", &W,&H);

	CharSymbol[SYMBOL_0] = ' ';
	CharSymbol[SYMBOL_1] = '1';
	CharSymbol[SYMBOL_2] = '2';
	CharSymbol[SYMBOL_3] = '3';
	CharSymbol[SYMBOL_4] = '4';
	CharSymbol[SYMBOL_5] = '5';
	CharSymbol[SYMBOL_6] = '6';
	CharSymbol[SYMBOL_7] = '7';
	CharSymbol[SYMBOL_8] = '8';
	CharSymbol[SYMBOL_NOT_CLICKED] = '.';
	CharSymbol[SYMBOL_BOMB] = 'B';
	CharSymbol[SYMBOL_RED_BOMB] = 'R';
	CharSymbol[SYMBOL_CROSS_BOMB] = 'X';
	CharSymbol[SYMBOL_MINE_MARK] = 'M';
	CharSymbol[SYMBOL_QUESTION_MARK_DEPRESSED] = '?';
	CharSymbol[SYMBOL_QUESTION_MARK] = '?';

	for (int i = 0; i < MAX_SQUARE_SYMBOLS; i++)
	{
		if (NULL == SquareSymbols[i])
		{
			//cerr << "ERROR LOADING FILE: " << i << endl;
		}
	}

	// Activate the win mine window.
	//oInput.ActivateExactWindow(WinMineWindowName);
	oInput.ActivateWindow(WinMineWindowName);
	// Get the dimensions of the winmine window.
	RECT Rect = oInput.GetLastActivatedWindowRect();
    WindowWidth = Rect.right - Rect.left;
    WindowHeight = Rect.bottom - Rect.top;
	WindowData = NULL;
    
	// The first cell is located at (15,96), the second one at (31,96).
    // The first cell in the second row is located at (15,112)
    // Thus, to advance in X, we must add 16. To advance in Y, we must add 16
    CellStartX = 15;
    CellStartY = 96;
    CellWidth = 16;
	CellHeight = 16;

	// Dimensions of the digits: [13,23] 
	DigitWidth = 13;
	DigitHeight = 23;
	NumberOfMines = 0;
	Seconds = 0;

	FaceWidth = 24;
	FaceHeight = 24;
	FaceStatus = FACE_OK;
    
    // <- 15 -> ... <- 10 ->
    GameWidth = WindowWidth - (15 + 10);
    GameHeight = WindowHeight - (96 + 10);
    GameCellsX = GameWidth / CellWidth;
    GameCellsY = GameHeight / CellHeight;
    //cerr << "GameCellsX: " << GameCellsX << ", GameCellsY: " << GameCellsY << endl;
	MineField = new int *[GameCellsY];
	for (int i = 0; i < GameCellsY; i++)
		MineField[i] = new int[GameCellsX];
}

WinMineReader::~WinMineReader()
{
	for (int i = 0; i < MAX_SQUARE_SYMBOLS; i++)
	{
		unsigned char **Symbol = SquareSymbols[i];
		for (int y = 0; y < CellHeight; y++)
			delete[] Symbol[y];
		delete[] Symbol;
	}

	for (int i = 0; i < MAX_DIGIT_SYMBOLS; i++)
	{
		unsigned char **Digit = DigitSymbols[i];
		for (int y = 0; y < DigitHeight; y++)
			delete[] Digit[y];
		delete[] Digit;
	}

	for (int i = 0; i < MAX_FACE_SYMBOLS; i++)
	{
		unsigned char **Face = FaceSymbols[i];
		for (int y = 0; y < FaceHeight; y++)
			delete[] Face[y];
		delete[] Face;
	}

	if (WindowData)
	{
		DeleteWindowData();
	}

	for (int i = 0; i < GameCellsY; i++)
		delete[] MineField[i];
	delete[] MineField;
}

void WinMineReader::RowColToXY(int Row, int Col, int *x, int *y)
{
	*x = Col*CellWidth  + CellStartX;
	*y = Row*CellHeight + CellStartY;
}

void WinMineReader::RowColToAbsoluteXY(int Row, int Col, int *x, int *y)
{
	RowColToXY(Row, Col, x, y);
	RECT Rect = oInput.GetLastActivatedWindowRect();
	*x += Rect.left + CellWidth/2;
	*y += Rect.top + CellHeight/2;
}

void WinMineReader::DeleteWindowData()
{
	if (WindowData)
	{
		for (int y = 0; y < WindowHeight; y++)
			delete[] WindowData[y];
		delete[] WindowData;
	}
	WindowData = NULL;
}

void WinMineReader::CaptureWindow()
{
	HBITMAP hBitmap = oInput.CaptureLastActivatedWindow();
	if (WindowData) DeleteWindowData();
	int W,H;
	WindowData = oInput.BitmapToGrayMatrix(hBitmap, &W, &H);
	DeleteObject(hBitmap);
}

int WinMineReader::ReadSymbolAt(int Row, int Col)
{
	if (NULL == WindowData) return SYMBOL_UNKNOWN;

	if (Row < 0 || Col < 0 ||
		Col > GameCellsX || Row > GameCellsY)
	{
		//cerr << "Invalid Row/Col: (" << Row << ", " << Col << ")" << endl;
		return SYMBOL_UNKNOWN;
	}

	int CellX,CellY;
	RowColToXY(Row, Col, &CellX,&CellY);
	if (CellX < 0 || CellY < 0)
	{
		return SYMBOL_UNKNOWN;
	}
	
	for (int i = 0; i < MAX_SQUARE_SYMBOLS; i++)
	{
		if (oInput.CompareMatricesAt(SquareSymbols[i],CellWidth,CellHeight, WindowData,WindowWidth,WindowHeight, CellX,CellY))
		{ 
			return i;
		}
	}
	return SYMBOL_UNKNOWN;
}

void WinMineReader::UpdateMineField()
{
	bool Error;
	do
	{
		CaptureWindow();

		Error = false;
		for (int row = 0; row < GameCellsY && !Error; row++)
			for (int col = 0; col < GameCellsX && !Error; col++)
			{
				MineField[row][col] = ReadSymbolAt(row, col);
				if (MineField[row][col] == SYMBOL_UNKNOWN)
				{
					Error = true;
				}					
			}

		if (Error)
		{
			//cerr << "Error while capturing image, retrying..." << endl;
		}
		else
		{
			// Read the digits that represent the number of mines left
			NumberOfMines = ReadDigitsAt(20,57);
			// Read the number of seconds
			Seconds = ReadDigitsAt(WindowWidth - 57,57);	
			// Read the face
			FaceStatus = ReadFace();
		}
	} while (Error);
}

int WinMineReader::ReadDigitsAt(int x, int y)
{
	// Read the first digit:
	int Digit0 = -1;
	for (int i = 0; i < MAX_DIGIT_SYMBOLS; i++)
	{
		if (oInput.CompareMatricesAt(DigitSymbols[i], DigitWidth, DigitHeight, WindowData,WindowWidth,WindowHeight, x + 13*2,y))
		{
			if (i != DIGIT_NULL)
				Digit0 = i;
			break;
		}
	}

	int Digit1 = -1;
	for (int i = 0; i < MAX_DIGIT_SYMBOLS; i++)
	{
		if (oInput.CompareMatricesAt(DigitSymbols[i], DigitWidth, DigitHeight, WindowData,WindowWidth,WindowHeight, x + 13,y))
		{
			if (i != DIGIT_NULL)
				Digit1 = i;
			break;
		}
	}

	int Digit2 = -1;
	for (int i = 0; i < MAX_DIGIT_SYMBOLS; i++)
	{
		if (oInput.CompareMatricesAt(DigitSymbols[i], DigitWidth, DigitHeight, WindowData,WindowWidth,WindowHeight, x,y))
		{
			if (i != DIGIT_NULL)
				Digit2 = i;
			break;
		}
	}

	//cerr << "Digits: " << Digit2 << ", " << Digit1 << ", " << Digit0 << endl;
	int Partial = Digit0 + (10*Digit1);
	if (Digit2 == DIGIT_MINUS)
		return -Partial;
	else
		return Partial + (100*Digit2);
}

int WinMineReader::ReadFace()
{
	// Coords for the happy face: (243,57)
	for (int i = 0; i < MAX_DIGIT_SYMBOLS; i++)
	{
		if (oInput.CompareMatricesAt(FaceSymbols[i], FaceWidth, FaceHeight, WindowData,WindowWidth,WindowHeight, WindowWidth/2 - 10,57))
		{
			return i;
		}
	}
	return FACE_UNKNOWN;
}

void WinMineReader::SendKey(WORD Key)
{
	ActivateWindow();
    oInput.TypeVirtualKey(Key);
}

void WinMineReader::ActivateWindow()
{ 
     oInput.ActivateWindow(WinMineWindowName); 
     Sleep(10);
}

void WinMineReader::MoveMouseTo(int Row, int Col)
{
	int x, y;
	RowColToAbsoluteXY(Row, Col, &x,&y);
    // Go to the randomly chosen start cell and click it!
	ActivateWindow();
    oInput.MoveMouseTo(x, y);
}

void WinMineReader::LeftClick()
{
	oInput.LeftClick();
}

void WinMineReader::RightClick()
{
	oInput.RightClick();
}
