/*
  Sample program to exemplify the use of the WinMineReader class. This program assumes "Buscaminas" is already running. If the
  window is in a different language, or if the window is not present at all, the behaviour of this program is undefined.

  The program captures the minefield and displays it. It then makes a pause and selects two random cells on the mine field where
  a left and right click are performed. After both clicks have been sent, the program captures the field again and displays the
  contents.

  Leo Reyes @ 2007.

  License/Disclaimer: You are hereby granted permission to do whatever you want with this code as long as you don't hold me responsible
  for any bad stuff that might happen to your computer/dog/significant other/whatever from the use of this code.
  
  Enjoy!
*/
#include <cstdlib>
#include <iostream>
#include <time.h>
#include <conio.h>
#include "SynthInput.h"
#include "WinMineReader.h"
#include <windows.h>

//#define PAUSE_ENABLED

using namespace std;

//WinMineReader oMineReader("Buscam");

enum FinishStateEnum {
     GAME_WON, GAME_LOST, PLAYING
};

class WinMinePlayer
{
  private:
    WinMineReader *oMineReader;
    int Rows, Cols, Iterations;
    bool FirstMove;
    FinishStateEnum FinishState;

  public:            
    WinMinePlayer(char *WindowName);
    ~WinMinePlayer();

    void ShowMineField();
    void ClickRandomCell();
    bool StartGame();
    bool DoMove();
    bool ScanBoardAndClick();
    FinishStateEnum GetFinishState();
    int GetIterations();
};

WinMinePlayer::WinMinePlayer(char *WindowName)
{
  oMineReader = new WinMineReader(WindowName);
  srand((int )time(NULL));
  Rows = oMineReader->GetRows();
  Cols = oMineReader->GetCols();
  FirstMove = true;
  Iterations = 0;
  FinishState = PLAYING;
}

WinMinePlayer::~WinMinePlayer()
{
  delete oMineReader;
}

void WinMinePlayer::ShowMineField()
{	
	int Symbol;	
	oMineReader->UpdateMineField();
	cerr << "Number of Mines: " << oMineReader->NumberOfMines << endl;
	cerr << "Seconds: " << oMineReader->Seconds << endl;
	switch (oMineReader->FaceStatus)
	{
		case FACE_OK : cerr << "Game OK" << endl; break;
		case FACE_NOT_OK : cerr << "Game Lost" << endl; break;
		case FACE_FINISH : cerr << "Game Won!!!" << endl; break;
		case FACE_UNKNOWN : cerr << "Face could not be read" << endl; break;
	}

	for (int i = 0; i < Rows; i++)
	{
		for (int j = 0; j < Cols; j++)
		{
			Symbol = oMineReader->MineField[i][j];
			if (Symbol < 0)
				cerr << "E ";
			else
				cerr << oMineReader->CharSymbol[Symbol] << " ";
		}
		cerr << endl;
	}
}

bool WinMinePlayer::StartGame()
{
   // Send an "F2" to the window, so that a new game begins
   oMineReader->SendKey(VK_F2);
}

void WinMinePlayer::ClickRandomCell()
{    
	//int RandomRow = rand() % oMineReader->GetRows();
	//int RandomCol = rand() % oMineReader->GetCols();
    int RandomRow; 
    int RandomCol; 

    oMineReader->UpdateMineField();
    bool KeepLooping = true;
    do
    {
      RandomRow = rand() % Rows;
      RandomCol = rand() % Cols;
      cerr << "Random cell: (" << RandomRow << ", " << RandomCol << ")" << endl;

      if (oMineReader->MineField[RandomRow][RandomCol] == SYMBOL_NOT_CLICKED)
      {
         cerr << "Clicking on unused board square" << endl;
	     oMineReader->MoveMouseTo(RandomRow, RandomCol);
	     oMineReader->LeftClick();
         KeepLooping = false;
      }
    } while (KeepLooping);

/*
	// Choose another random cell and right-click it.
	RandomRow = rand() % oMineReader.GetRows();
	RandomCol = rand() % oMineReader.GetCols();
    cerr << "Random cell: (" << RandomRow << ", " << RandomCol << ")" << endl;

	oMineReader.MoveMouseTo(RandomRow, RandomCol);
	oMineReader.RightClick();
*/
	ShowMineField();
}

bool WinMinePlayer::ScanBoardAndClick()
{
    int Symbol;
    bool CellClicked = false;
    for (int i = 0; i < Rows; i++)
	{
		for (int j = 0; j < Cols; j++)
		{
			Symbol = oMineReader->MineField[i][j];
			if (Symbol > 0 && Symbol < 9)
            {
               int FreeCellCount = 0;
               int MinesToAccountFor = Symbol;
               cerr << "==================================" << endl;
               cerr << "Scanning cell: [" << i << ", " << j << "]" << endl;
               cerr << "MinesToAccountFor: " << MinesToAccountFor << endl;
               cerr << "FreeCellCount: " << FreeCellCount << endl;

               for (int ii = i - 1; ii <= i + 1; ii++)
               {
                  for (int jj = j - 1; jj <= j + 1; jj++)
                  {
                     if (ii >= 0 && ii < Rows && 
                         jj >= 0 && jj < Cols &&
                         !(ii == i && jj == j))
                     {
                        cerr << "\tCell being scanned: [" << ii << ", " << jj << "]" << endl;
                        if (oMineReader->MineField[ii][jj] == SYMBOL_NOT_CLICKED)
                        {
                           FreeCellCount++;                   
                           cerr << "\tFreeCellCount: " << FreeCellCount << endl;
                        }
                        else if (oMineReader->MineField[ii][jj] == SYMBOL_MINE_MARK)
                        {
                           MinesToAccountFor--;
                           cerr << "\tMinesToAccountFor: " << MinesToAccountFor << endl;
                        }
                     }
                  }
               }
               cerr << "Finished scanning." << endl;
               cerr << "MinesToAccountFor: " << MinesToAccountFor << endl;
               cerr << "FreeCellCount: " << FreeCellCount << endl;
               cerr << "==================================" << endl;
               if (MinesToAccountFor > 0 && 
                   MinesToAccountFor == FreeCellCount)
               {
                      cerr << "MinesToAccountFor == FreeCellCount (" << FreeCellCount << ")" << endl;
                      cerr << "Position: [" << i << ", " << j << "]" << endl;
                      cerr << "Right-clicking on the surrounding cells" << endl;
                      #ifdef PAUSE_ENABLED
                      system("PAUSE");
                      #endif
                      for (int ii = i - 1; ii <= i + 1; ii++)
                      {
                          for (int jj = j - 1; jj <= j + 1; jj++)
                          {
                              if (ii >= 0 && ii < Rows && 
                                  jj >= 0 && jj < Cols &&
                                  !(ii == i && jj == j))
                              {
                                 if (oMineReader->MineField[ii][jj] == SYMBOL_NOT_CLICKED)
                                 {
                                    cerr << "Clicking on unused board square: [" << ii << ", " << jj << "]" << endl;
                                    #ifdef PAUSE_ENABLED
                                    system("PAUSE");
                                    #endif
	                                oMineReader->MoveMouseTo(ii, jj);
	                                oMineReader->RightClick();
                                    CellClicked = true;
                                 }
                              }
                          }
                      }
                      oMineReader->UpdateMineField();
               } // MinesToAccountFor
               else if (MinesToAccountFor == 0 && FreeCellCount > 0)
               {
                 // All the mines have been accounted for (by marks laid on the board); but
                 // there are still free cells. We should left-click those cells to enable 
                 // further processing.
                 cerr << "MinesToAccountFor == 0 && FreeCellCount == " << FreeCellCount << endl;
                 cerr << "Position: [" << i << ", " << j << "]" << endl;
                 cerr << "LEFT-clicking on the surrounding unused cells" << endl;
                 #ifdef PAUSE_ENABLED
                 system("PAUSE");
                 #endif
                 for (int ii = i - 1; ii <= i + 1; ii++)
                 {
                   for (int jj = j - 1; jj <= j + 1; jj++)
                   {
                     if (ii >= 0 && ii < Rows && 
                         jj >= 0 && jj < Cols &&
                         !(ii == i && jj == j))
                        {
                           if (oMineReader->MineField[ii][jj] == SYMBOL_NOT_CLICKED)
                           {
                              cerr << "Left-clicking on unused board square: [" << ii << ", " << jj << "]" << endl;
                              #ifdef PAUSE_ENABLED
                              system("PAUSE");
                              #endif
	                          oMineReader->MoveMouseTo(ii, jj);
	                          oMineReader->LeftClick();
                              CellClicked = true;
                           }
                        }
                   }
                 }
                 oMineReader->UpdateMineField();
               }
            }            
		}
		cerr << endl;
	}
    return CellClicked;
}

FinishStateEnum WinMinePlayer::GetFinishState()
{
    return FinishState;
}

int WinMinePlayer::GetIterations()
{
    return Iterations;
}

// Returns true if the game is over, either way.
bool WinMinePlayer::DoMove()
{
     bool Finished = false;
     Iterations++;
     cerr << "***************************************" << endl;
     cerr << "Iteration: " << Iterations << endl;
     // Check the current status of the game
     oMineReader->UpdateMineField();
	 switch (oMineReader->FaceStatus)
	 {
		case FACE_NOT_OK : cerr << "Game Lost" << endl; 
             if (FirstMove)
             {
                FirstMove = false;
                cerr << "Restarting game" << endl;
                StartGame();
                oMineReader->UpdateMineField();
                break;
             }
             else
             {
                 FinishState = GAME_LOST;
                 return true;
             }
		case FACE_FINISH : 
             if (FirstMove)
             {
                FirstMove = false;
                cerr << "Starting new game" << endl;
                StartGame();
                oMineReader->UpdateMineField();
                break;
             }
             else
             {
                cerr << "Game Won!!!" << endl;
                FinishState = GAME_WON;
                return true;
             }
		case FACE_UNKNOWN : cerr << "Face could not be read" << endl;
             break;
	 }

     ShowMineField();

     // We can only reach this point if the game is going OK.
     // We must now check what's the current situation on the board. The first
     // step is to check any position on the board where there are numbers and
     // see if any of these numbers give us clues as to where hidden mines might
     // be.
     bool Clicked = ScanBoardAndClick();

     if (!Clicked)
     {
        // If no position on the board could be clicked due to the reasoning engine,
        // then click anywhere...
        ClickRandomCell();
     }

     FirstMove = false;
     return Finished;
}

int main(int argc, char *argv[])
{
    WinMinePlayer *oPlayer = new WinMinePlayer("Buscam");
	//ShowMineField();
    oPlayer->ShowMineField();

    #ifdef PAUSE_ENABLED
	system("PAUSE");
    #endif

    bool Finished = false;
	do
    {
      Finished = oPlayer->DoMove();
      #ifdef PAUSE_ENABLED
      system("PAUSE");
      #endif
    } while (!Finished);

    if (oPlayer->GetFinishState() == GAME_WON)
    {
       cerr << "Game won!!!" << endl;
    }
    cerr << "Iterations done: " << oPlayer->GetIterations() << endl;
    system("PAUSE");
	return 0;
}

